<?php

/********************************************************************************
 *
 *	pngcat.php	ver.0.1        by MALU 2006/11/11
 *
 *
 *	Supported PNG format:
 *		8bit / palette / non-interlace ONLY
 *		
 *	Function:
 *		boolean pngcat(string png_file, string hex_num [, number num_max = 10]);
 *		
 *		Returns:
 *			TRUE  - Success
 *			FALSE - Failure
 *		
 *	Example:
 *		require('pngcat.php');
 *		
 *		$count = 123;
 *		pngcat('./image/numbers.png', sprintf('%05d', $count));
 *			// Output> 00123
 *		
 *		// If you want to print whitespace(or any) instead of zero:
 *		pngcat('./image/nums_and_space.png', sprintf("%'a5d", $count), 10 + 1);
 *			// Output> __123
 *		
 *		// Are you hex freak ? :-)
 *		pngcat('./image/hexnums.png', sprintf('%05x', $count), 0x10);
 *			// Output> 0007B
 */


/*	If you are not allowed to use zlib, set following FALSE. */
/*	Then, you need to prepare 'Non-Compressed' PNG file.     */
	$use_zlib = TRUE;



function adler32($dat) {
	$s1 = 1;
	$s2 = 0;
	$len = strlen($dat);
	for ($i = 0; $i < $len; $i++) {
		$s1 = ($s1 + ord($dat[$i])) % 65521;
		$s2 = ($s2 + $s1) % 65521;
	}
	return $s2 << 16 | $s1;
}

function png_signature() {
	return "\x89PNG\r\n\x1A\n";
}

function png_chunk($name, $data = '') {
	return pack('N', strlen($data)) . $name . $data . pack('N', crc32($name . $data));
}

function png_IHDR($cx, $cy, $type = 3, $bpp = 8) {
	return png_chunk('IHDR', pack('NNCCCCC', $cx, $cy, $bpp, $type, 0, 0, 0));
}

function png_IDAT($img) {
	global $use_zlib;
	if ($use_zlib && function_exists('gzdeflate')) {
		return png_chunk('IDAT', "\x78\x9C" . gzdeflate($img) . pack('N', adler32($img)));
	}
	$len = strlen($img);
	if ($len < 0x10000) {
		return png_chunk('IDAT', "\x78\x9C\x01" . pack('vv', $len, ~$len) . $img . pack('N', adler32($img)));
	}
	$buf = '';
	$ptr = 0;
	while ($ptr + 0xFFFF < $len) {
		$buf .= "\0\xFF\xFF\0\0" . substr($img, $ptr, 0xFFFF);
		$ptr += 0xFFFF;
	}
	$len -= $ptr;
	return png_chunk('IDAT', "\x78\x9C$buf\x01" . pack('vv', $len, ~$len) . substr($img, $ptr) . pack('N', adler32($img)));
}

function png_IEND() {
	return "\0\0\0\0IEND\xAE\x42\x60\x82";
}

function pngcat($png_file, $hex_num, $num_max = 10) {
	global $use_zlib;
	
	$fp = @fopen($png_file, 'rb');
	if (!$fp) return FALSE;
	flock($fp, LOCK_SH);
	if (fread($fp, 8) != png_signature()) { fclose($fp); return FALSE; }
	
	$cx   = 0;
	$cy   = 0;
	$sRGB = FALSE;
	$gAMA = FALSE;
	$cHRM = FALSE;
	$PLTE = FALSE;
	$tRNS = FALSE;
	$bits = FALSE;
	
	do {
		$ary  = unpack('Nlen', fread($fp, 4));
		$len  = $ary['len'];
		$name = fread($fp, 4);
		$data = $len ? fread($fp, $len) : '';
		$ary  = unpack('Ncrc', fread($fp, 4));
		$crc  = $ary['crc'];
		switch ($name) {
		case 'IHDR':
			$ary  = unpack('Ncx/Ncy/Cbpp/Ctype/Ccomp/Cfltr/Cintl', $data);
			$cx   = $ary['cx'];
			$cy   = $ary['cy'];
			if ($cx <= 0 || $cy <= 0 || $ary['bpp'] != 8 || $ary['type'] != 3 ||
			    $ary['comp'] || $ary['fltr'] || $ary['intl']) { fclose($fp); return FALSE; }
			break;
		case 'sRGB':
			$sRGB = $data;
			break;
		case 'gAMA':
			$gAMA = $data;
			break;
		case 'cHRM':
			$cHRM = $data;
			break;
		case 'PLTE':
			$PLTE = $data;
			break;
		case 'tRNS':
			$tRNS = $data;
			break;
		case 'IDAT':
			$bits = substr($data, 2, -4);
			break;
		case 'IEND':
			break 2;
		}
	//	if (crc32($name . $data) != $crc) { fclose($fp); return FALSE; }
	} while (!feof($fp));
	fclose($fp);
	
	if (!$cx || !$cy || !$PLTE || !$bits) return FALSE;
	
	if ($use_zlib && function_exists('gzinflate')) {
		$bits = gzinflate($bits);
		if ($bits === FALSE) return FALSE;
	} else {
		$tmp = '';
		$ptr = 0;
		while (($b = ord($bits[$ptr])) != 1) {
			if ($b != 0) return FALSE; // Without zlib, support only Non-Compressed
			$ary  = unpack('vlen', substr($bits, $ptr + 1, 2));
			$len  = $ary['len'];
			$tmp .= substr($bits, $ptr + 5, $len);
			$ptr += 5 + $len;
		}
		$bits = $tmp . substr($bits, $ptr + 5);
	}
	
	$paeth = create_function('$l, $u, $p', '
		$t = $l + $u - $p;
		return min(abs($t - $l), abs($t - $u), abs($t - $p));
	');
	for ($y = 0; $y < $cy; $y++) {
		$line = (1 + $cx) * $y;
		switch (ord($bits[$line])) {
		case 0:
			break;
		case 1:
			for ($x = 1; $x < $cx; $x++) {
				$t = $line + 1 + $x; $l = $t - 1;
				$bits[$t] = chr((ord($bits[$t]) + ord($bits[$l])) & 0xFF);
			}
			break;
		case 2:
		//	if ($y == 0) return FALSE;
			for ($x = 0; $x < $cx; $x++) {
				$t = $line + 1 + $x; $u = $t - (1 + $cx);
				$bits[$t] = chr((ord($bits[$t]) + ord($bits[$u])) & 0xFF);
			}
			break;
		case 3:
		//	if ($y == 0) return FALSE;
			for ($x = 0; $x < $cx; $x++) {
				$t = $line + 1 + $x; $l = $t - 1; $u = $t - (1 + $cx);
				$bits[$t] = chr((ord($bits[$t]) + ((ord($bits[$l]) + ord($bits[$u])) >> 1)) & 0xFF);
			}
			break;
		case 4:
		//	if ($y == 0) return FALSE;
			for ($x = 0; $x < $cx; $x++) {
				$t = $line + 1 + $x; $l = $t - 1; $u = $t - (1 + $cx); $p = $t - (1 + $cx) - 1;
				$bits[$t] = chr((ord($bits[$t]) + $paeth(ord($bits[$l]), ord($bits[u]), ord($bits[p]))) & 0xFF);
			}
			break;
		}
	}
	
	$w   = (int)($cx / $num_max);
	$buf = '';
	
	$len = strlen($hex_num);
	for ($y = 0; $y < $cy; $y++) {
		$buf .= "\0";
		for ($i = 0; $i < $len; $i++) {
			$n = hexdec($hex_num[$i]);
			if ($n >= $num_max) return FALSE;
			$buf .= substr($bits, (1 + $cx) * $y + 1 + $w * $n, $w);
		}
	}
	
	if (function_exists('mb_http_output')) mb_http_output('pass');
	header('Content-type: image/png');
	
	print png_signature();
	print png_IHDR($w * $len, $cy);
	if ($sRGB !== FALSE) print png_chunk('sRGB', $sRGB);
	if ($gAMA !== FALSE) print png_chunk('gAMA', $gAMA);
	if ($cHRM !== FALSE) print png_chunk('cHRM', $cHRM);
	print png_chunk('PLTE', $PLTE);
	if ($tRNS !== FALSE) print png_chunk('tRNS', $tRNS);
	print png_IDAT($buf);
	print png_IEND();
	
	return TRUE;
}

?>